/*
HDD Explorer AppList Editor 
Copyright (c) 2006 Bruno Freitas
bootsector@ig.com.br
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

import java.util.Collection;
import java.util.Vector;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class FrmManAplicacao extends JDialog implements ActionListener {
    
    private JPanel pnlDadosAplicacao, pnlOkFechar;
    private JLabel lblTitulo, lblAutor, lblDescricao, lblExecutavel;
    private JTextField txtTitulo, txtAutor, txtDescricao, txtExecutavel;
    private JList lstArquivos;
    private DefaultListModel lmArquivos;
    private JButton btnOk, btnFechar;
    private Aplicacao aplicacao;
    private boolean incluir;
    
    public FrmManAplicacao(Frame owner, String titulo, Aplicacao aplicacao, boolean incluir) {
        super(owner, titulo, true);
        
        this.aplicacao = aplicacao;
        this.incluir = incluir;
        
        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        
        // Painel Dados da Aplicao
        pnlDadosAplicacao = new JPanel(new GridLayout(8, 1));
        pnlDadosAplicacao.setBorder(new TitledBorder("Application Info:"));
        
        lblTitulo = new JLabel("Title:");
        pnlDadosAplicacao.add(lblTitulo);
        txtTitulo = new JTextField(10);
        txtTitulo.setText(aplicacao.getTitulo());
        pnlDadosAplicacao.add(txtTitulo);
        
        lblAutor = new JLabel("Author:");
        pnlDadosAplicacao.add(lblAutor);
        txtAutor = new JTextField(10);
        txtAutor.setText(aplicacao.getAutor());
        pnlDadosAplicacao.add(txtAutor);
        
        lblDescricao = new JLabel("Description (use \\n to line-break):");
        pnlDadosAplicacao.add(lblDescricao);
        txtDescricao = new JTextField(10);
        txtDescricao.setText(aplicacao.getDescricao().replace("\n", "\\n"));
        pnlDadosAplicacao.add(txtDescricao);
        
        lblExecutavel = new JLabel("Filename:");
        pnlDadosAplicacao.add(lblExecutavel);
        txtExecutavel = new JTextField(10);
        txtExecutavel.setText(aplicacao.getExecutavel());
        pnlDadosAplicacao.add(txtExecutavel);
        
        
        pnlOkFechar = new JPanel(new GridLayout(1, 2));
        btnOk = new JButton("OK");
        btnOk.setMnemonic('O');
        btnOk.setIcon(new ImageIcon(getClass().getResource("res/img/ok.png")));
        btnOk.addActionListener(this);
        pnlOkFechar.add(btnOk);
        
        btnFechar = new JButton("Close");
        btnFechar.setMnemonic('C');
        btnFechar.setIcon(new ImageIcon(getClass().getResource("res/img/cancelar.png")));
        btnFechar.addActionListener(this);
        pnlOkFechar.add(btnFechar);
        
        container.add(pnlDadosAplicacao, BorderLayout.NORTH);
        container.add(pnlOkFechar, BorderLayout.SOUTH);
        
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        
        setSize(400, 250);
        setResizable(false);
        setLocationRelativeTo(owner);
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btnFechar) {
            setVisible(false);
        }
        
        if (e.getSource() == btnOk) {
            if (txtTitulo.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "You must type application's title!", "Warning", JOptionPane.WARNING_MESSAGE);
                txtTitulo.requestFocus();
                return;
            }
            
            if (txtAutor.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "You must type application's author!", "Warning", JOptionPane.WARNING_MESSAGE);
                txtAutor.requestFocus();
                return;
            }
            
            if (txtDescricao.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "You must type application's description!", "Warning", JOptionPane.WARNING_MESSAGE);
                txtDescricao.requestFocus();
                return;
            }
            
            if (txtExecutavel.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "You must type application's filename!", "Warning", JOptionPane.WARNING_MESSAGE);
                txtExecutavel.requestFocus();
                return;
            }
            
            aplicacao.setTitulo(txtTitulo.getText());
            aplicacao.setAutor(txtAutor.getText());
            aplicacao.setDescricao(txtDescricao.getText());
            aplicacao.setExecutavel(txtExecutavel.getText().toUpperCase().replace('\\', '/'));
            setVisible(false);
        }
    }
}
